﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;

namespace howto_image_circle
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private Bitmap BaseImage = Properties.Resources.cake;
        private Bitmap BgImage = null;
        private Color BgColor = Color.White;

        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                BaseImage = LoadBitmapUnlocked(ofdImage.FileName);
            }
        }

        private void mnuFileBackgroundColor_Click(object sender, EventArgs e)
        {
            cdBackgroundColor.Color = BgColor;
            if (cdBackgroundColor.ShowDialog() == DialogResult.OK)
            {
                BgColor = cdBackgroundColor.Color;
                Bitmap bm = new Bitmap(16, 16);
                using (Graphics gr = Graphics.FromImage(bm))
                {
                    gr.Clear(BgColor);
                    gr.DrawRectangle(Pens.Black, 0, 0, 15, 15);
                }
                mnuFileBackgroundColor.Image = bm;
                BgImage = null;
            }
        }

        private void mnuFileBackgroundImage_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                BgImage = LoadBitmapUnlocked(ofdImage.FileName);
                txtPicWidth.Text = BgImage.Width.ToString();
                txtPicHeight.Text = BgImage.Height.ToString();
            }
        }

        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    SaveImage(picResult.Image, sfdImage.FileName);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        // If we have a background and base image, make the circle.
        private void MakeCircle()
        {
            // Get the parameters.
            int pic_width, pic_height, img_width, img_height, num_images;
            double offset_multiple, initial_rotation, radius;
            if (ParametersInvalid(
                out pic_width, out pic_height,
                out img_width, out img_height,
                out offset_multiple, out initial_rotation,
                out num_images, out radius))
                    return;

            // Size the result PictureBox and the form.
            picResult.ClientSize = new Size(pic_width, pic_height);
            int wid = Math.Min(panScroll.Left + picResult.Width + panScroll.Left, 800);
            int hgt = Math.Min(panScroll.Top + picResult.Width + panScroll.Left, 800);
            if (wid < ClientSize.Width) wid = ClientSize.Width;
            if (hgt < ClientSize.Height) hgt = ClientSize.Height;
            ClientSize = new Size(wid, hgt);

            // Make the result bitmap.
            Bitmap bm = new Bitmap(pic_width, pic_height);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                gr.InterpolationMode = InterpolationMode.High;
                gr.SmoothingMode = SmoothingMode.AntiAlias;
                if (BgImage != null)
                {
                    // Draw the background image.
                    using (Brush brush = new TextureBrush(BgImage))
                    {
                        gr.FillRectangle(brush, 0, 0, pic_width, pic_height);
                    }
                }
                else
                {
                    // Fill the background with the background color.
                    gr.Clear(BgColor);
                }

                // Draw the circle.
                DrawImageCircle(gr, BaseImage,
                    pic_width, pic_height,
                    img_width, img_height,
                    offset_multiple, initial_rotation,
                    num_images, radius);
            }

            // Display the result.
            picResult.Image = bm;
        }

        // Get the parameters. If any of the parameters is
        // missing or invalid, tell the user and return true.
        private bool ParametersInvalid(
            out int pic_width, out int pic_height,
            out int img_width, out int img_height,
            out double theta_offset, out double initial_rotation,
            out int num_images, out double radius)
        {
            pic_width = 0;
            pic_height = 0;
            img_width = 0;
            img_height = 0;
            theta_offset = 0;
            initial_rotation = 0;
            num_images = 6;
            radius = 100;

            if (!int.TryParse(txtPicWidth.Text, out pic_width) || (pic_width <= 0))
            {
                MessageBox.Show("Picture width must be greater than zero.");
                txtPicWidth.Focus();
                return true;
            }
            if (!int.TryParse(txtPicHeight.Text, out pic_height) || (pic_height <= 0))
            {
                MessageBox.Show("Picture width must be greater than zero.");
                txtPicHeight.Focus();
                return true;
            }
            if (!int.TryParse(txtImgWidth.Text, out img_width) || (img_width <= 0))
            {
                MessageBox.Show("Image width must be greater than zero.");
                txtImgWidth.Focus();
                return true;
            }
            if (!int.TryParse(txtImgHeight.Text, out img_height) || (img_height <= 0))
            {
                MessageBox.Show("Image width must be greater than zero.");
                txtImgHeight.Focus();
                return true;
            }
            if (!double.TryParse(txtOffsetMultiple.Text, out theta_offset))
            {
                MessageBox.Show("Theta offset must be a number.");
                txtOffsetMultiple.Focus();
                return true;
            }
            if (!double.TryParse(txtInitialRotation.Text, out initial_rotation))
            {
                MessageBox.Show("Initial rotation must be a number.");
                txtInitialRotation.Focus();
                return true;
            }
            if (!int.TryParse(txtNumImages.Text, out num_images) || (num_images < 1))
            {
                MessageBox.Show("Number of images must be at least one.");
                txtNumImages.Focus();
                return true;
            }
            if (!double.TryParse(txtRadius.Text, out radius) || (radius < 0))
            {
                MessageBox.Show("Radius must be at least zero.");
                txtNumImages.Focus();
                return true;
            }

            return false;
        }

        // Draw the circle of images.
        private void DrawImageCircle(Graphics gr,
            Bitmap image,
            int pic_width, int pic_height,
            int img_width, int img_height,
            double offset_multiple,
            double initial_rotation,
            int num_images, double radius)
        {
            GraphicsState state = gr.Save();

            // Get the picture's center.
            float cx = pic_width / 2f;
            float cy = pic_height / 2f;

            // Adjust the image size to preserve aspect ratio.
            double scale_x = (double)img_width / image.Width;
            double scale_y = (double)img_width / image.Height;
            double scale = Math.Min(scale_x, scale_y);
            img_width = (int)(image.Width * scale);
            img_height = (int)(image.Height * scale);

            // Get the image's source rectangle.
            RectangleF src_rect = new RectangleF(
                0, 0, image.Width, image.Height);

            // Make destination points to center the image at the origin.
            PointF[] dest_points =
            {
                new PointF(-img_width / 2f, -img_height / 2f),
                new PointF( img_width / 2f, -img_height / 2f),
                new PointF(-img_width / 2f,  img_height / 2f),
            };

            // Loop through the images.
            double dtheta = 360 / num_images;
            double theta = dtheta * offset_multiple;
            double angle = initial_rotation + theta;
            for (int i = 0; i < num_images; i++)
            {
                // Get the point where the image's center should be drawn.
                double x = cx + radius * Math.Cos(theta * Math.PI / 180);
                double y = cy + radius * Math.Sin(theta * Math.PI / 180);
                PointF point = new PointF((float)x,(float)y);

                // Rotate and then translate to (x, y).
                gr.ResetTransform();
                gr.RotateTransform((float)angle);
                gr.TranslateTransform((float)x, (float)y, MatrixOrder.Append);

                // Draw the image.
                gr.DrawImage(image, dest_points, src_rect, GraphicsUnit.Pixel);

                theta += dtheta;
                angle += dtheta;
            }

            gr.Restore(state);
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            try
            {
                using (Bitmap bm = new Bitmap(file_name))
                {
                    return new Bitmap(bm);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return null;
            }
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            try
            {
                string extension = Path.GetExtension(filename);
                switch (extension.ToLower())
                {
                    case ".bmp":
                        image.Save(filename, ImageFormat.Bmp);
                        break;
                    case ".exif":
                        image.Save(filename, ImageFormat.Exif);
                        break;
                    case ".gif":
                        image.Save(filename, ImageFormat.Gif);
                        break;
                    case ".jpg":
                    case ".jpeg":
                        image.Save(filename, ImageFormat.Jpeg);
                        break;
                    case ".png":
                        image.Save(filename, ImageFormat.Png);
                        break;
                    case ".tif":
                    case ".tiff":
                        image.Save(filename, ImageFormat.Tiff);
                        break;
                    default:
                        throw new NotSupportedException(
                            "Unknown file extension " + extension);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnDraw_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;
            MakeCircle();
            Cursor = Cursors.Default;
        }
    }
}
